package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.model.entities.CloneableEntity;
import java.io.Serializable;
import java.util.List;
import java.util.Map;
import javax.annotation.PostConstruct;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import org.apache.log4j.Logger;
import org.primefaces.component.datatable.DataTable;

public abstract class DataTableController<EntityType extends CloneableEntity> implements Serializable
{

    private static final Logger logger = Logger.getLogger(DataTableController.class.getName());


    protected EntityType current = null;

    private DataModel listDataModel = null;
    private DataTable listDataTable = null;
    private boolean listDataModelReset = true;
    private List<EntityType> filteredObjectList = null;
    private List<EntityType> selectedObjectList = null;

    
    public DataTableController() {
    }

    @PostConstruct
    public void initialize() {
    }
    
    public abstract String getEntityTypeName();

//    public abstract String getCurrentEntityInstanceName();

    public EntityType getCurrent() {
        return current;
    }

    public void setCurrent(EntityType current) {
        this.current = current;
    }

    public DataModel createListDataModel() {
        return new ListDataModel(findAll());
    }

    public DataModel getListDataModel() {
        if (listDataModel == null) {
            listDataModel = createListDataModel();
        }
        return listDataModel;
    }

    public abstract List<EntityType> findAll();

    public void resetList() {
        logger.debug("Resetting list");
        listDataModel = null;
        listDataTable = null;
        listDataModelReset = true;
        filteredObjectList = null;
        selectedObjectList = null;
        current = null;
    }

   public DataTable getListDataTable() {
        if (listDataTable == null) {
            logger.debug("Recreating data table");
            listDataTable = new DataTable();
        }
        return listDataTable;
    }

    public void setListDataTable(DataTable listDataTable) {
        this.listDataTable = listDataTable;
    }

    public boolean isAnyListFilterSet() {
        if (listDataTable == null) {
            return false;
        }
        Map<String, Object> filterMap = listDataTable.getFilters();
        for (Object filter : filterMap.values()) {
            if ((String)filter != null && !((String)filter).isEmpty()) {
                return true;
            }
        }
        return false;
    }

    public List<EntityType> getFilteredObjectList() {
        return filteredObjectList;
    }

     public void setFilteredObjectList(List<EntityType> filteredObjectList) {
        this.filteredObjectList = filteredObjectList;
    }

    public List<EntityType> getFilteredItems() {
        return filteredObjectList;
    }

    public void setFilteredItems(List<EntityType> filteredItems) {
        this.filteredObjectList = filteredItems;
    }
    
    public void resetFilterList() {
        filteredObjectList = null;
    }
    
    public boolean isListDataModelReset() {
        if (listDataModelReset) {
            listDataModelReset = false;
            return true;
        }
        return false;
    }

    public List<EntityType> getSelectedObjectList() {
        return selectedObjectList;
    }
    
    public void setSelectedObjectList(List<EntityType> selectedObjectList) {
        this.selectedObjectList = selectedObjectList;
    }
    
}
