package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.exceptions.InvalidDate;
import gov.anl.aps.dm.portal.exceptions.MissingProperty;
import gov.anl.aps.dm.portal.model.entities.Experiment;
import gov.anl.aps.dm.portal.model.beans.ExperimentFacade;
import java.util.List;

import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import org.apache.log4j.Logger;

@Named("experimentController")
@SessionScoped
public class ExperimentController extends CrudEntityController<Experiment, ExperimentFacade>
{

    private static final Logger logger = Logger.getLogger(ExperimentController.class.getName());

    @EJB
    private ExperimentFacade experimentFacade;

    public ExperimentController() {
    }

    @Override
    protected ExperimentFacade getFacade() {
        return experimentFacade;
    }

    @Override
    protected Experiment createEntityInstance() {
        return new Experiment();
    }

    @Override
    public String getEntityTypeName() {
        return "experiment";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public List<Experiment> getAvailableItems() {
        return super.getAvailableItems();
    }

    @Override
    public String prepareEdit(Experiment experiment) {
        return super.prepareEdit(experiment);
    }

    @Override
    public void prepareEntityInsert(Experiment experiment) throws ObjectAlreadyExists, InvalidDate, MissingProperty {
        Experiment existingExperiment = experimentFacade.findByName(experiment.getName());
        if (existingExperiment != null) {
            throw new ObjectAlreadyExists("Experiment " + experiment.getName() + " already exists.");
        }

        verifyExperiment(experiment);
        logger.debug("Inserting new experiment " + experiment.getName());
    }

    @Override
    public void prepareEntityUpdate(Experiment experiment) throws DmPortalException {
        verifyExperiment(experiment);        
        logger.debug("Updating experiment " + experiment.getName());                
    }

     @Override
     protected String getObjectAlreadyExistMessage(Experiment experiment)  {
        if (experiment == null)
            return null;
        return "Experiment " + experiment.getName() + " already exists."; 
    }

    private void verifyExperiment(Experiment experiment) throws InvalidDate, MissingProperty {       
        if (experiment.getExperimentType() == null) {
            throw new MissingProperty("Experiment Type is missing.");
        }
        
        if ((experiment.getStartDate() != null) && (experiment.getEndDate() != null) && (experiment.getEndDate().before(experiment.getStartDate()))) {
            throw new InvalidDate("Experiment end date is before start date.");
        }
    }
   

    @FacesConverter(forClass = Experiment.class)
    public static class ExperimentControllerConverter implements Converter
    {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            ExperimentController controller = (ExperimentController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "experimentController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof Experiment) {
                Experiment o = (Experiment) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + Experiment.class.getName());
            }
        }

    }

}