package gov.anl.aps.dm.portal.controllers;

import gov.anl.aps.dm.portal.exceptions.DmPortalException;
import gov.anl.aps.dm.portal.exceptions.ObjectAlreadyExists;
import gov.anl.aps.dm.portal.model.entities.ExperimentType;
import gov.anl.aps.dm.portal.model.beans.ExperimentTypeFacade;
import java.util.List;

import javax.ejb.EJB;
import javax.inject.Named;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.FacesConverter;
import org.apache.log4j.Logger;

@Named("experimentTypeController")
@SessionScoped
public class ExperimentTypeController extends CrudEntityController<ExperimentType, ExperimentTypeFacade>
{

    private static final Logger logger = Logger.getLogger(ExperimentTypeController.class.getName());

    @EJB
    private ExperimentTypeFacade experimentTypeFacade;

    public ExperimentTypeController() {
    }

    @Override
    protected ExperimentTypeFacade getFacade() {
        return experimentTypeFacade;
    }

    @Override
    protected ExperimentType createEntityInstance() {
        return new ExperimentType();
    }

    @Override
    public String getEntityTypeName() {
        return "experimentType";
    }

    @Override
    public String getCurrentEntityInstanceName() {
        if (getCurrent() != null) {
            return getCurrent().getName();
        }
        return "";
    }

    @Override
    public List<ExperimentType> getAvailableItems() {
        return super.getAvailableItems();
    }

    @Override
    public String prepareEdit(ExperimentType experimentType) {
        return super.prepareEdit(experimentType);
    }

    @Override
    public void prepareEntityInsert(ExperimentType experimentType) throws ObjectAlreadyExists {
        ExperimentType existingExperimentType = experimentTypeFacade.findByName(experimentType.getName());
        if (existingExperimentType != null) {
            throw new ObjectAlreadyExists("Experiment type " + experimentType.getName() + " already exists.");
        }
        logger.debug("Inserting new experiment type " + experimentType.getName());
    }

    @Override
    public void prepareEntityUpdate(ExperimentType experimentType) throws DmPortalException {
    }
    
      @Override
     protected String getObjectAlreadyExistMessage(ExperimentType experimentType)  {
        if (experimentType == null)
            return null;
        return "Experiment Type " + experimentType.getName() + " already exists."; 
    }

    @FacesConverter(forClass = ExperimentType.class)
    public static class ExperimentTypeControllerConverter implements Converter
    {

        @Override
        public Object getAsObject(FacesContext facesContext, UIComponent component, String value) {
            if (value == null || value.length() == 0) {
                return null;
            }
            ExperimentTypeController controller = (ExperimentTypeController) facesContext.getApplication().getELResolver().
                    getValue(facesContext.getELContext(), null, "experimentTypeController");
            return controller.getEntity(getKey(value));
        }

        java.lang.Integer getKey(String value) {
            java.lang.Integer key;
            key = Integer.valueOf(value);
            return key;
        }

        String getStringKey(java.lang.Integer value) {
            StringBuilder sb = new StringBuilder();
            sb.append(value);
            return sb.toString();
        }

        @Override
        public String getAsString(FacesContext facesContext, UIComponent component, Object object) {
            if (object == null) {
                return null;
            }
            if (object instanceof ExperimentType) {
                ExperimentType o = (ExperimentType) object;
                return getStringKey(o.getId());
            }
            else {
                throw new IllegalArgumentException("object " + object + " is of type " + object.getClass().getName() + "; expected type: " + ExperimentType.class.getName());
            }
        }

    }

}